import spacy
import os

try:
    nlp = spacy.load("en_core_web_trf")
except:
    print("Downloading spaCy NLP model...")
    print("This may take a few minutes and it's one time process...")
    os.system("pip install https://huggingface.co/spacy/en_core_web_trf/resolve/main/en_core_web_trf-any-py3-none-any.whl")
    nlp = spacy.load("en_core_web_trf")


def perform_ner(*args, **kwargs):
    query = kwargs['query']
    # Process the input text with spaCy NLP model
    doc = nlp(query)

    # Extract named entities and categorize them
    entities = [(entity.text, entity.label_) for entity in doc.ents]

    return entities


if __name__ == "__main__":
    # Example input text
    input_text = "I want to buy a new iPhone 12 Pro Max from Apple."

    # Perform NER on input text
    entities = perform_ner(query=input_text)

    # Print the extracted entities
    print(entities)



import os
import json
import logging
import requests

try:
    from utils import input_output
    from brain import intent_classification
    from brain import ner
    from features_manager import action_map
    from brain.auth import verify_user
except:
    from JarvisAI.utils import input_output
    from JarvisAI.brain import intent_classification
    from JarvisAI.brain import ner
    from JarvisAI.features_manager import action_map
    from JarvisAI.brain.auth import verify_user


def action_handler(intent, query, api_key):
    intent = intent.replace("_", " ")
    if intent in action_map:
        logging.info(f"Intent {intent} matched. Calling action {action_map[intent]}")
        entities = ner.perform_ner(query=query)
        try:
            ans = action_map[intent](query=query, intent=intent, entities=entities, input_output_fun=input_output,
                                     api_key=api_key)
            if ans == "Entity not found":
                intent = "others"
                print(
                    "Entity not found, calling action for intent 'others'. If you are not upgraded to our premium plan, "
                    "please upgrade to use this feature.")
                ans = action_map[intent](query=query, intent=intent, entities=entities, input_output_fun=input_output,
                                         api_key=api_key)
                return ans
            return ans
        except Exception as e:
            ans = action_map[intent](query=query, intent=intent, entities=entities, input_output_fun=input_output,
                                     api_key=api_key)
            if ans == "Entity not found":
                intent = "others"
                print(
                    "Entity not found, calling action for intent 'others'. If you are not upgraded to our premium plan, "
                    "please upgrade to use this feature.")
                ans = action_map[intent](query=query, intent=intent, entities=entities, input_output_fun=input_output,
                                         api_key=api_key)
                return ans
            return ans
    else:
        logging.info(f"Intent {intent} not found in action map.")
        return "Sorry, I don't know how to handle this intent."


# function to add new actions to the actions_map dictionary
def add_action(intent: str, action: object):
    """Add a new action to the action map.
    @param intent: (String) The intent to be mapped to the action.
    @param action: (Object) The function to call when the intent is matched.
    @return: (String) The message to be displayed to the user.
    """
    try:
        intent = intent.replace("_", " ")
        action_map[intent] = action

        if not os.path.exists('actions.json'):
            url = "https://raw.githubusercontent.com/Dipeshpal/Jarvis_AI/master/JarvisAI/JarvisAI/actions.json"
            data = requests.get(url).json()
            with open('actions.json', 'w') as f:
                json.dump(data, f)

        with open("actions.json", "r") as f:
            actions = json.load(f)

        # Check for duplicates
        duplicates = [i for i, act in enumerate(actions) if act["intent"] == intent]
        if len(duplicates) > 1:
            for i in duplicates[1:]:
                actions.pop(i)
            logging.warning(f"Duplicate actions found for intent {intent}. Only the first action will be used.")
            print(f"Found {len(duplicates)} duplicates for intent '{intent}'. Deleted all but the first.")

        # Check if the action already exists
        action_exists = False
        for i, act in enumerate(actions):
            if act["intent"] == intent:
                action_exists = True
                action_index = i
                break

        if action_exists:
            print(f"Previous examples for intent '{intent}': {actions[action_index]['example']}")
            overwrite = input(f"Intent '{intent}' already exists. Do you want to overwrite? (y/n)")
            if overwrite.lower() == "y":
                num_examples = int(input("How many examples do you want to add? (maximum 3): "))
                examples = []
                for i in range(num_examples):
                    examples.append(input(f"Enter example {i + 1} for intent {intent}: "))
                actions[action_index] = {
                    "intent": intent,
                    "example": examples
                }
            else:
                return "Intent not overwritten."
        else:
            num_examples = int(input("How many examples do you want to add? (maximum 10): "))
            examples = []
            for i in range(num_examples):
                examples.append(input(f"Enter example {i + 1} for intent {intent}: "))
            actions.append({
                "intent": intent,
                "example": examples
            })

        # Write the updated actions back to the file
        with open("actions.json", "w") as f:
            json.dump(actions, f, indent=4)

        logging.info(f"Action for intent {intent} has been added/updated. Train new model to use this action.")
        print(f"Action for intent {intent} has been added/updated. Train new model to use this action.")
        return f"Action for intent {intent} has been added/updated. Train new model to use this action."
    except Exception as e:
        logging.error(f"Error adding action: {e}")
        raise f"Error adding action: {e}"


class JarvisAI(input_output.JarvisInputOutput):
    def __init__(self, input_mechanism='text', output_mechanism='text',
                 google_speech_api_key=None, backend_tts_api='pyttsx3',
                 use_whisper_asr=False, display_logs=False,
                 api_key=None):
        super().__init__(input_mechanism=input_mechanism, output_mechanism=output_mechanism,
                         google_speech_api_key=google_speech_api_key, backend_tts_api=backend_tts_api,
                         use_whisper_asr=use_whisper_asr, duration_listening=5, display_logs=display_logs,
                         api_key=api_key)

        if os.path.exists('jarvis.log'):
            os.remove('jarvis.log')

        self.display_logs = display_logs
        if not self.display_logs:
            logging.basicConfig(filename='jarvis.log', level=logging.DEBUG)
        else:
            logging.basicConfig(level=logging.DEBUG)

        if not os.path.exists('actions.json'):
            url = "https://raw.githubusercontent.com/Dipeshpal/Jarvis_AI/master/JarvisAI/JarvisAI/actions.json"
            data = requests.get(url).json()
            with open('actions.json', 'w') as f:
                json.dump(data, f)

    def handle_input(self):
        try:
            if self.input_mechanism == 'text':
                return self.text_input()
            elif self.input_mechanism == 'voice':
                return self.voice_input()
            else:
                available_input_mechanisms = ['text', 'voice']
                logging.error(f"Invalid input mechanism: {self.input_mechanism}. Available input mechanisms are: "
                              f"{available_input_mechanisms}")
                raise Exception("Invalid input mechanism. Available input mechanisms are: {available_input_mechanisms}")
        except Exception as e:
            logging.exception(f"An error occurred while handling input. Error: {e}")
            return f"An error occurred while handling input. Error: {e}"

    def handle_output(self, text):
        try:
            if self.output_mechanism == 'text':
                self.text_output(text)
            elif self.output_mechanism == 'voice':
                self.voice_output(text=text)
            elif self.output_mechanism == 'both':
                self.text_output(text)
                self.voice_output(text=text)
            else:
                available_output_mechanisms = ['text', 'voice', 'both']
                logging.error(f"Invalid output mechanism: {self.output_mechanism}. Available output mechanisms are: "
                              f"{available_output_mechanisms}")
                raise f"Invalid output mechanism: {self.output_mechanism}. Available output mechanisms are: " \
                      f"{available_output_mechanisms}"
        except Exception as e:
            logging.exception(f"An error occurred while handling output. Error: {e}")
            self.handle_output(f"An error occurred while handling output. Error: {e}")

    def take_action(self, intent, query):
        try:
            # if not os.path.exists('actions.json'):
            #     # TODO: Add a default actions.json file
            #     logging.error("actions.json file not found.")
            #     return "actions.json file not found."
            #
            # # load the JSON file containing the list of available actions and their respective commands
            # with open('actions.json', 'r') as f:
            #     actions = json.load(f)

            # check if the intent matches any of the available actions
            # for action in actions:
            # intent, _ = intent_classification.classify_intent(secret_key=self.api_key, text=query)
            # if the intent matches, do the action
            try:
                return action_handler(intent, query, self.api_key)
            except Exception as e:
                logging.exception(f"An error occurred while performing action. Error: {e}")
                self.handle_output(f"An error occurred while performing action. Error: {e}")

            print(f"Intent {intent} not found in actions.json.")
            # if no action is matched, return a default message
            return "Sorry, I don't know how to handle this intent."
        except Exception as e:
            logging.exception(f"An error occurred while taking action. Error: {e}")
            return f"An error occurred while taking action. Error: {e}"

    # don't change this function, do not try to remove verify_user otherwise Jarvis will not work
    @verify_user
    def start(self):
        while True:
            try:
                query = self.handle_input()
                if query == "" or query is None:
                    continue
                if query == 'exit':
                    self.handle_output("Exiting...")
                    break
                else:
                    # NOTE: The query is passed to the intent classification function to get the intent
                    intent, _ = intent_classification.classify_intent(secret_key=self.api_key, text=query)
                    # print(f"Intent: {intent}")
                    # intent = intent.replace("_", " ")
                    # PATCH BELOW for date and time if-else
                    if 'time' in intent.replace("_", " "):
                        print(f"Intent: date / {intent}")
                    else:
                        print(f"Intent: {intent}")
                    logging.debug(f"Input: {query}, Intent: {intent}")
                    response = self.take_action(intent, query)
                    self.handle_output(response)
            except Exception as e:
                logging.exception(f"An error occurred while running Jarvis. Error: {e}")
                self.handle_output(f"An error occurred while running Jarvis. Error: {e}")
                raise f"An error occurred while running Jarvis. Error: {e}"


if __name__ == "__main__":
    def custom_function(*args, **kwargs):
        command = kwargs.get('query')
        entities = kwargs.get('entities')
        print(entities)
        # write your code here to do something with the command
        # perform some tasks # return is optional
        return command + ' Executed'


    jarvis = JarvisAI(input_mechanism='text', output_mechanism='text',
                      google_speech_api_key=None, backend_tts_api='pyttsx3',
                      use_whisper_asr=False, display_logs=False,
                      api_key='527557f2-0b67-4500-8ca0-03766ade589a')
    add_action("general", custom_function)
    jarvis.start()



import shutup
import requests
import json

shutup.please()


def verify_user(func):
    def inner(self, *args, **kwargs):
        with open('api_key.txt', 'r') as f:
            api_key = f.read()
        url = f'https://jarvisai.in/check_secret_key?secret_key={api_key}'
        response = requests.get(url)
        if response.status_code == 200:
            status = json.loads(response.content)
            if status:
                print("Authentication Successful")
                return func(self, *args, **kwargs)  # Return the result of calling func
            else:
                print("Authentication Failed. Please check your API key.")
                exit()
        else:
            print(f'Error: {response.status_code}')
            exit()

    return inner


if __name__ == "__main__":
    data = verify_user("527557f2-0b67-4500-8ca0-03766ade589a")
    print(data, type(data))



import webbrowser


def get_places_near_me(*args, **kwargs):
    inp_command = kwargs.get("query")
    map_base_url = f"https://www.google.com/maps/search/{inp_command}"
    webbrowser.open(map_base_url)

    return "Opening Google Maps"


if __name__ == "__main__":
    print(get_places_near_me("nearest coffee shop"))



import requests


def get_weather(*args, **kwargs):
    print("Getting weather")
    query = kwargs.get("query")
    entities = kwargs.get("entities")
    city = "Indore"
    if len(entities) == 0:
        city = [entity[0] for entity in entities if entity[1] == "GPE"][0]
        if len(city) == 0:
            return "Unable to in which city you want to know weather"
    geo_url = f"https://geocoding-api.open-meteo.com/v1/search?name={city}&count=1"
    geo_data = requests.get(geo_url).json()
    lat = geo_data['results'][0]["latitude"]
    lon = geo_data['results'][0]["longitude"]
    weather_url = f"https://api.open-meteo.com/v1/forecast?latitude={lat}&longitude={lon}&hourly=temperature_2m"
    weather_data = requests.get(weather_url).json()
    temp = weather_data["hourly"]["temperature_2m"][-1]
    return f"The temperature in {city} is {temp} degrees Celsius."


if __name__ == "__main__":
    print(get_weather(entities=[("London", "GPE")]))



import pyjokes


def tell_me_joke(*args, **kwargs):
    """
    Function to tell a joke
    Read https://pyjok.es/api/ for more details
    :return: str
    """
    lang = kwargs.get("lang", "en")
    cat = kwargs.get("cat", "neutral")
    return pyjokes.get_joke(language=lang, category=cat)


if __name__ == '__main__':
    print(tell_me_joke())



try:
    from utils.speech_to_text.speech_to_text import speech_to_text_google
    from utils.text_to_speech.text_to_speech import text_to_speech
    from utils.speech_to_text.speech_to_text_whisper import speech_to_text_whisper
except:
    from JarvisAI.utils.speech_to_text.speech_to_text import speech_to_text_google
    from JarvisAI.utils.text_to_speech.text_to_speech import text_to_speech
    from JarvisAI.utils.speech_to_text.speech_to_text_whisper import speech_to_text_whisper


class JarvisInputOutput:
    def __init__(self, input_mechanism='text', output_mechanism='text', logging=None,
                 google_speech_api_key=None, google_speech_recognition_input_lang='en',
                 duration_listening=5, backend_tts_api='pyttsx3',
                 use_whisper_asr=False, display_logs=False,
                 api_key=None):
        self.input_mechanism = input_mechanism
        self.output_mechanism = output_mechanism
        self.google_speech_api_key = google_speech_api_key
        self.google_speech_recognition_input_lang = google_speech_recognition_input_lang
        self.duration_listening = duration_listening
        self.backend_tts_api = backend_tts_api
        self.logging = logging
        self.use_whisper_asr = use_whisper_asr
        self.display_logs = display_logs
        self.api_key = api_key
        with open('api_key.txt', 'w') as f:
            f.write(api_key)
        # print("JarvisInputOutput initialized")
        # print(f"Input mechanism: {self.input_mechanism}")
        # print(f"Output mechanism: {self.output_mechanism}")
        # print(f"Google Speech API Key: {self.google_speech_api_key}")
        # print(f"Backend TTS API: {self.backend_tts_api}")

    def text_input(self):
        if self.input_mechanism == 'text':
            return input("Enter your query: ")
        else:
            self.logging.exception("Invalid input mechanism")
            raise ValueError("Invalid input mechanism")

    def text_output(self, text):
        if self.output_mechanism == 'text' or self.output_mechanism == 'both':
            print(text)
        else:
            self.logging.exception("Invalid output mechanism")
            raise ValueError("Invalid output mechanism")

    def voice_input(self, *args, **kwargs):
        if self.input_mechanism == 'voice':
            if self.use_whisper_asr:
                if self.display_logs:
                    print("Using Whisper ASR")
                command, status = speech_to_text_whisper(duration=self.duration_listening)
            else:
                if self.display_logs:
                    print("Using Google ASR")
                command, status = speech_to_text_google(input_lang=self.google_speech_recognition_input_lang,
                                                        key=self.google_speech_api_key,
                                                        duration=self.duration_listening)
            print(f"You Said: {command}")
            if status:
                return command
            else:
                return None
        else:
            self.logging.exception("Invalid input mechanism")
            raise ValueError("Invalid input mechanism")

    def voice_output(self, *args, **kwargs):
        if self.output_mechanism == 'voice' or self.output_mechanism == 'both':
            text = kwargs.get('text', None)
            text_to_speech(text=text, lang='en', backend_tts_api=self.backend_tts_api)
        else:
            self.logging.exception("Invalid output mechanism")
            raise ValueError("Invalid output mechanism")



import requests
from googlesearch import search
import wikipedia
import html2text
from bs4 import BeautifulSoup
from markdown import markdown
import re


def search_google_description(query):
    try:
        ans = search(query, advanced=True, num_results=5)

        desc = ''
        for i in ans:
            desc += i.description

        return desc
    except:
        return ''


def query_pages(query):
    return list(search(query))


def markdown_to_text(markdown_string):
    """ Converts a markdown string to plaintext """

    # md -> html -> text since BeautifulSoup can extract text cleanly
    html = markdown(markdown_string)

    # remove code snippets
    html = re.sub(r'<pre>(.*?)</pre>', ' ', html)
    html = re.sub(r'<code>(.*?)</code >', ' ', html)

    # extract text
    soup = BeautifulSoup(html, "html.parser")
    for e in soup.find_all():
        if e.name not in ['p']:
            e.unwrap()
    text = ''.join([i.strip() for i in soup.findAll(text=True)])
    return text


def format_text(text):
    text = markdown_to_text(text)
    text = text.replace('\n', ' ')
    return text


def search_google(query):
    try:
        def query_to_text(query):
            html_conv = html2text.HTML2Text()
            html_conv.ignore_links = True
            html_conv.escape_all = True
            text = []
            for link in query_pages(query)[0:3]:
                try:
                    headers = {
                        'User-Agent': 'Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36'}

                    req = requests.get(link, headers=headers)
                    text.append(html_conv.handle(req.text))
                    text[-1] = format_text(text[-1])
                except:
                    pass
            return text

        return query_to_text(query)
    except:
        return ''


def search_wiki(query):
    try:
        ans = wikipedia.summary(query, sentences=2)
        return ans
    except:
        return ''


def search_all(query, advance_search=False):
    combined = search_google_description(query) + "\n" + search_wiki(query)
    if advance_search:
        combined += "\n" + ' '.join(search_google(query))
    return combined


def try_to_get_response(*args, **kwargs):
    api_key = kwargs.get("api_key")
    query = kwargs.get('query')
    context = search_all(query, advance_search=False)
    headers = {
        'accept': 'application/json',
        'content-type': 'application/x-www-form-urlencoded',
    }
    params = {
        'secret_key': api_key,
        'text': query,
        'context': context,
    }
    response = requests.post('https://www.jarvisai.in/chatbot_premium_api', params=params, headers=headers)
    if response.status_code == 200:
        return response.json()['data']
    else:
        return response.json().get("message", "Server is facing some issues. Please try again later.")


def premium_chat(*args, **kwargs):
    # try 3 times to call try_to_get_response(*args, **kwargs) function and if it fails then return error message
    for i in range(3):
        try:
            return try_to_get_response(*args, **kwargs)
        except Exception as e:
            pass
    return "Server is facing some issues. Please try again later."
    # try:
    #     try_to_get_response(*args, **kwargs)
    # except Exception as e:
    #     return f"An error occurred while performing premium_chat, connect with developer. Error: {e}"


if __name__ == "__main__":
    print(premium_chat(query="who is naredra modi", api_key='ae44cc6e-0d5c-45c1-b8a3-fe412469510f'))



import speedtest  # pip install speedtest-cli

try:
    st = speedtest.Speedtest()
except:
    print("Please check your internet connection.")
    pass


def download_speed():
    down = round(st.download() / 10 ** 6, 2)
    return down


def upload_speed():
    up = round(st.upload() / 10 ** 6, 2)
    return up


def ping():
    servernames = []
    st.get_servers(servernames)
    results = st.results.ping
    return results


def speed_test(*args, **kwargs):
    try:
        print("Checking internet speed. Please wait...")
        # print('Download Speed: ' + str(download_speed()) + 'MB/s')
        # print('Upload Speed: ' + str(upload_speed()) + ' MB/s')
        # print('Ping: ' + str(ping()) + ' ms')
        return "Download Speed: " + str(download_speed()) + "MB/s" + "\n Upload Speed: " + str(
            upload_speed()) + " MB/s" + "\n Ping: " + str(ping()) + " ms"
    except Exception as e:
        print(e)
        return "Error in internet speed test"


if __name__ == '__main__':
    print(speed_test())



import datetime
import cv2
import os


def click_pic(*args, **kwargs):
    try:
        t = datetime.datetime.now()
        #     Taking a video from the webcam
        camera = cv2.VideoCapture(0)
        #     Taking first 20 frames of the video
        for i in range(20):
            return_value, image = camera.read()
        if not os.path.exists("photos"):
            os.mkdir("photos")
        #     Using 20th frame as the picture and now saving the image as the time in seconds,minute,hour,day and month of the year
        # Giving the camera around 20 frames to adjust to the surroundings for better picture quality
        cv2.imwrite(f"photos/{t.second, t.minute, t.hour, t.day, t.month}_photo.png", image)
        #     As soon as the image is saved we will stop recording
        del camera
        print(f"Photo taken: photos/{t.second, t.minute, t.hour, t.day, t.month}_photo.png")
        return "Photo taken"
    except Exception as e:
        return "Error: " + str(e) + "\n Unable to take photo"


# Calling the photo_with_python function
if __name__ == "__main__":
    click_pic()



import datetime


def greet(*args, **kwargs):
    time = datetime.datetime.now().hour
    if time < 12:
        return "Hi, Good Morning"
    elif 12 <= time < 18:
        return "Hi, Good Afternoon"
    else:
        return "Hi, Good Evening"


def goodbye(*args, **kwargs):
    return "Goodbye"


import webbrowser


def news(*args, **kwargs):
    """
    This method will open the browser and show the news "https://thetechport.in/"
    :return: list / bool
    """
    try:
        url = "https://thetechport.in/"
        webbrowser.open(url)
        return True
    except Exception as e:
        print(e)
        return False


def show_me_some_tech_news():
    try:
        url = "https://thetechport.in/"
        webbrowser.open(url)
        return True
    except Exception as e:
        print(e)
        return False


def show_me_some_tech_videos():
    try:
        url = "https://www.youtube.com/c/TechPortOfficial"
        webbrowser.open(url)
        return True
    except Exception as e:
        print(e)
        return False


if __name__ == "__main__":
    print(news())
    print(show_me_some_tech_news())



import cv2
import mediapipe as mp
from math import hypot
from ctypes import cast, POINTER
from comtypes import CLSCTX_ALL
from pycaw.pycaw import AudioUtilities, IAudioEndpointVolume
import numpy as np


def start_volume_control(*args, **kwargs):
    cap = cv2.VideoCapture(0)  # Checks for camera

    mpHands = mp.solutions.hands  # detects hand/finger
    hands = mpHands.Hands()  # complete the initialization configuration of hands
    mpDraw = mp.solutions.drawing_utils

    # To access speaker through the library pycaw
    devices = AudioUtilities.GetSpeakers()
    interface = devices.Activate(IAudioEndpointVolume._iid_, CLSCTX_ALL, None)
    volume = cast(interface, POINTER(IAudioEndpointVolume))
    volbar = 400
    volper = 0

    volMin, volMax = volume.GetVolumeRange()[:2]

    while True:
        success, img = cap.read()  # If camera works capture an image
        img = cv2.flip(img, 1)
        imgRGB = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)  # Convert to rgb
        cv2.putText(img, f"Press SPACE to stop", (20, 80), cv2.FONT_ITALIC, 1, (0, 255, 98), 3)

        # Collection of gesture information
        results = hands.process(imgRGB)  # completes the image processing.

        lmList = []  # empty list
        if results.multi_hand_landmarks:  # list of all hands detected.
            # By accessing the list, we can get the information of each hand's corresponding flag bit
            for handlandmark in results.multi_hand_landmarks:
                for id, lm in enumerate(handlandmark.landmark):  # adding counter and returning it
                    # Get finger joint points
                    h, w, _ = img.shape
                    cx, cy = int(lm.x * w), int(lm.y * h)
                    lmList.append([id, cx, cy])  # adding to the empty list 'lmList'
                mpDraw.draw_landmarks(img, handlandmark, mpHands.HAND_CONNECTIONS)

        if lmList != []:
            # getting the value at a point
            # x      #y
            x1, y1 = lmList[4][1], lmList[4][2]  # thumb
            x2, y2 = lmList[8][1], lmList[8][2]  # index finger
            # creating circle at the tips of thumb and index finger
            cv2.circle(img, (x1, y1), 13, (255, 0, 0), cv2.FILLED)  # image #fingers #radius #rgb
            cv2.circle(img, (x2, y2), 13, (255, 0, 0), cv2.FILLED)  # image #fingers #radius #rgb
            cv2.line(img, (x1, y1), (x2, y2), (255, 0, 0), 3)  # create a line b/w tips of index finger and thumb

            length = hypot(x2 - x1, y2 - y1)  # distance b/w tips using hypotenuse
            # from numpy we find our length,by converting hand range in terms of volume range ie b/w -63.5 to 0
            vol = np.interp(length, [30, 350], [volMin, volMax])
            volbar = np.interp(length, [30, 350], [400, 150])
            volper = np.interp(length, [30, 350], [0, 100])

            print(vol, int(length))
            volume.SetMasterVolumeLevel(vol, None)

            # Hand range 30 - 350
            # Volume range -63.5 - 0.0
            # creating volume bar for volume level

            cv2.rectangle(img, (50, 150), (85, 400), (0, 0, 255),
                          4)  # vid ,initial position ,ending position ,rgb ,thickness
            cv2.rectangle(img, (50, int(volbar)), (85, 400), (0, 0, 255), cv2.FILLED)
            cv2.putText(img, f"{int(volper)}%", (10, 40), cv2.FONT_ITALIC, 1, (0, 255, 98), 3)
            # tell the volume percentage ,location,font of text,length,rgb color,thickness
        # flip the image
        cv2.imshow('Image', img)  # Show the video
        if cv2.waitKey(1) & 0xff == ord(' '):  # By using spacebar delay will stop
            break

    cap.release()  # stop cam
    cv2.destroyAllWindows()  # close window
    return "Volume control stopped"


if __name__ == '__main__':
    start_volume_control()



import whisper
import sounddevice
from scipy.io.wavfile import write
import time

model = whisper.load_model("base")


def recorder(second=5):
    fs = 16000
    print("Recording.....")
    record_voice = sounddevice.rec(int(second * fs), samplerate=fs, channels=2)
    sounddevice.wait()
    write("./recording.wav", fs, record_voice)
    print("Finished.....")
    time.sleep(1)


def speech_to_text_whisper(duration=5):
    try:
        recorder(second=duration)
        result = model.transcribe("./recording.wav")
        return result.get("text"), True
    except Exception as e:
        print(e)
        return e, False


if __name__ == "__main__":
    speech_to_text_whisper()



import datetime


def date_time(*args, **kwargs):
    query = kwargs['query']
    if 'time' in query:
        return datetime.datetime.now().strftime("%H:%M:%S")
    elif 'date' in query:
        return datetime.datetime.now().strftime("%d/%m/%Y")
    else:
        return "Sorry, I don't know how to handle this intent."



import os
import requests
import json


def check_local_intent(text):
    """Check if the intent of a text string is available locally.

    Args:
        text (str): The text string to check.

    Returns:
        str: The intent of the text string.
    """
    try:
        if not os.path.exists('actions.json'):
            return None
        else:
            with open('actions.json', 'r') as f:
                actions = json.load(f)
            # Below is sample actions.json file
            # [
            #   {
            #     "intent": "greet and hello hi kind of things",
            #     "example": []
            #   },
            #   {
            #     "intent": "goodbye",
            #     "example": ['bye', 'goodbye', 'see you later']
            #   }
            # ]
            # if text in example then return intent
            for action in actions:
                if text in action['example']:
                    return action['intent']
                if action['intent'] in text:
                    return action['intent']
                if action['example'] == "":
                    continue
    except Exception as e:
        return None


def try_to_classify_intent(secret_key, text):
    """Classify the intent of a text string using the Wit.ai API.

       Args:
           text (str): The text string to classify.

       Returns:
           str: The intent of the text string.
       """
    try:
        intent = check_local_intent(text)
        if intent is not None:
            return intent, 1.0
    except Exception as e:
        pass

    try:
        url = f'https://jarvisai.in/intent_classifier?secret_key={secret_key}&text={text}'
        response = requests.get(url)
        data = response.json()
        if data['status'] == 'success':
            return data['data'][0], data['data'][1]
    except Exception as e:
        return e, None


def classify_intent(secret_key, text):
    for i in range(3):
        try:
            return try_to_classify_intent(secret_key, text)
        except Exception as e:
            pass

if __name__ == "__main__":
    intent, _ = classify_intent('99f605ce-5bf9-4e80-93a3-f367df65aa27', "custom function")
    print(intent, _)



import pywhatkit


def send_email(*args, **kwargs):
    try:
        my_email = input("Enter your email address: ")
        my_password = input("Enter your password: ")
        mail_to = input("Enter the email address you want to send to: ")
        subject = input("Enter the subject of the email: ")
        content = input("Enter the content of the email: ")
        print("Sending email...")
        pywhatkit.send_mail(email_sender=my_email,
                            password=my_password,
                            subject=subject,
                            message=content,
                            email_receiver=mail_to)
        print("Email sent!")
        return 'Email sent!'
    except Exception as e:
        print(e)
        return 'Email not sent!, Check Error. or Check secure apps is enabled'


if __name__ == "__main__":
    send_email(None)



import webbrowser
import re


def website_opener(*args, **kwargs):
    input_text = kwargs.get("query")
    domain = input_text.lower().split(" ")[-1]
    extension = re.search(r"[.]", domain)
    if not extension:
        if not domain.endswith(".com"):
            domain = domain + ".com"
    try:
        url = 'https://www.' + domain
        webbrowser.open(url)
        return True
    except Exception as e:
        print(e)
        return False


if __name__ == '__main__':
    website_opener("facebook")



import setuptools
from setuptools import find_namespace_packages

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="JarvisAI",
    version="4.8",
    author="Dipesh",
    author_email="dipeshpal17@gmail.com",
    description="JarvisAI is python library to build your own AI virtual assistant with natural language processing.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Dipeshpal/Jarvis_AI",
    include_package_data=True,
    packages=find_namespace_packages(include=['JarvisAI.*', 'JarvisAI']),
    install_requires=['numpy', 'gtts', 'playsound', 'pyscreenshot', "opencv-python",
                      'SpeechRecognition', 'pyjokes', 'wikipedia', 'scipy', 'lazyme',
                      "requests", "pyttsx3", "spacy==3.5.0", 'pywhatkit', 'speedtest-cli',
                      'pytube', 'pycountry', 'playsound', 'pyaudio', 'mediapipe==0.8.11',
                      'pycaw', 'openai-whisper', 'shutup', 'sounddevice', 'html2text==2020.1.16',
                      'wikipedia==1.4.0', 'Markdown==3.4.1', 'markdown2==2.4.8',
                      'lxml==4.9.2', 'googlesearch-python==1.2.3'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    project_urls={
        'Official Website': 'https://jarvisai.in',
        'Documentation': 'https://github.com/Dipeshpal/Jarvis_AI',
        'Donate': 'https://www.buymeacoffee.com/dipeshpal',
        'Say Thanks!': 'https://youtube.com/techportofficial',
        'Source': 'https://github.com/Dipeshpal/Jarvis_AI',
        'Contact': 'https://www.dipeshpal.in/social',
    },
)



from pytube import YouTube
import os


def download_yt_video(*arg, **kwargs):
    ytURL = input("Enter the URL of the YouTube video: ")
    yt = YouTube(ytURL)
    try:
        print("Downloading...")
        yt.streams.filter(progressive=True, file_extension="mp4").order_by("resolution")[-1].download()
    except:
        return "ERROR | Please try again later"
    return f"Download Complete | Saved at {os.getcwd()}"


if __name__ == "__main__":
    download_yt_video()



import wikipedia
import re


def tell_me_about(*args, **kwargs):
    # topic = kwargs.get("query")
    entities = kwargs.get("entities")
    if len(entities) == 0:
        return "Entity not found"
    li = ['EVENT', 'FAC', 'GPE', 'LANGUAGE', 'LAW', 'LOC', 'MONEY', 'NORP', 'ORDINAL', 'ORG',
          'PERCENT', 'PERSON', 'PRODUCT', 'TIME', 'WORK_OF_ART']
    topic = [entity[0] for entity in entities if entity[1] in li][0]
    try:
        ny = wikipedia.page(topic)
        res = str(ny.content[:500].encode('utf-8'))
        res = re.sub('[^a-zA-Z.\d\s]', '', res)[1:]
        return res
    except Exception as e:
        print(e)
        return False


if __name__ == '__main__':
    import spacy
    import os

    try:
        nlp = spacy.load("en_core_web_trf")
    except:
        print("Downloading spaCy NLP model...")
        print("This may take a few minutes and it's one time process...")
        os.system(
            "pip install https://huggingface.co/spacy/en_core_web_trf/resolve/main/en_core_web_trf-any-py3-none-any.whl")
        nlp = spacy.load("en_core_web_trf")


    def perform_ner(*args, **kwargs):
        query = kwargs['query']
        # Process the input text with spaCy NLP model
        doc = nlp(query)

        # Extract named entities and categorize them
        entities = [(entity.text, entity.label_) for entity in doc.ents]

        return entities


    query = "tell me about Narendra Modi"
    # Perform NER on input text
    entities = perform_ner(query=query)

    print(tell_me_about(query=query, entities=entities))



try:
    from features.date_time import date_time
    from features.greet import greet, goodbye
    from features.joke import tell_me_joke
    from features.click_photo import click_pic
    from features.covid_cases import check_command_is_for_covid_cases
    from features.games import play_games
    from features.iambored import get_me_suggestion
    from features.internet_speed_test import speed_test
    from features.news import news
    from features.places_near_me import get_places_near_me
    from features.screenshot import take_screenshot
    from features.send_email import send_email
    from features.tell_me_about import tell_me_about
    from features.volume_controller import start_volume_control
    from features.weather import get_weather
    from features.website_open import website_opener
    from features.whatsapp_message import send_whatsapp_message
    from features.youtube_play import yt_play
    from features.youtube_video_downloader import download_yt_video
    from brain import chatbot_premium
except Exception as e:
    from .features.date_time import date_time
    from .features.greet import greet, goodbye
    from .features.joke import tell_me_joke
    from .features.click_photo import click_pic
    from .features.covid_cases import check_command_is_for_covid_cases
    from .features.games import play_games
    from .features.iambored import get_me_suggestion
    from .features.internet_speed_test import speed_test
    from .features.news import news
    from .features.places_near_me import get_places_near_me
    from .features.screenshot import take_screenshot
    from .features.send_email import send_email
    from .features.tell_me_about import tell_me_about
    from .features.volume_controller import start_volume_control
    from .features.weather import get_weather
    from .features.website_open import website_opener
    from .features.whatsapp_message import send_whatsapp_message
    from .features.youtube_play import yt_play
    from .features.youtube_video_downloader import download_yt_video
    from .brain import chatbot_premium


def show_what_can_i_do(*args, **kwargs):
    print("I can do following things:")
    for key in action_map.keys():
        print(key)


action_map = {
    "asking time": date_time,
    "asking date": date_time,
    "greet and hello hi kind of things": greet,
    "goodbye": goodbye,
    "tell me joke": tell_me_joke,
    "tell me about": tell_me_about,  # TODO: improve this
    "i am bored": get_me_suggestion,
    "volume control": start_volume_control,
    "tell me news": news,
    "click photo": click_pic,
    "places near me": get_places_near_me,
    "play on youtube": yt_play,
    "play games": play_games,
    "what can you do": show_what_can_i_do,
    "send email": send_email,
    "download youtube video": download_yt_video,
    "asking weather": get_weather,
    "take screenshot": take_screenshot,
    "open website": website_opener,
    "send whatsapp message": send_whatsapp_message,
    "covid cases": check_command_is_for_covid_cases,
    "check internet speed": speed_test,
    "others": chatbot_premium.premium_chat,
}
